from flask import send_file
from flask_restful import Resource, request
from lyrebird.mock import context

class SnapshotExportFromEvent(Resource):
    def post(self):
        file = context.application.data_manager.export_snapshot_from_event(request.json)
        return send_file(file)

class SnapshotExportFromDM(Resource):
    def post(self):
        node_id = request.json.get("nodeId")
        file = context.application.data_manager.export_snapshot_from_dm(node_id)
        return send_file(file)



import re
from lyrebird.log import get_logger

logger = get_logger()


class JSONPath:

    @staticmethod
    def search(root, path:str):
        """ Find JSON object in object (a ``list`` or ``dict`` JSON object) file by JSONPath

        ``root`` <list> or <dict> an object containing data

        ``path`` <str> describe JSONPath of the target data, and must follow rules:
        1. Start with `$`
        2. Fuzzy search is not supported

        """

        if not path or not isinstance(path, str) or not isinstance(root, (list, dict)):
            return

        # split by `.` and drop `.`, split by `[ ]` and keep `[ ]`
        # EXAMPLE
        # path = '$.data[0][1].name'
        # keys = ['$', 'data', '[0]', '[1]', 'name']
        pattern = r'(?:\.)|(?=\[.*\])'
        keys = re.split(pattern, path)
        if not keys or not len(keys):
            return

        if keys[0] != '$':
            logger.warning(f'JSONPath error! JSONPath {path} does not start with `$`!')
            return

        result = []
        JSONPath._search_iterator(root, keys[1:], result)
        return result

    @staticmethod
    def _search_iterator(root, prop_keys, result):
        current_key = prop_keys[0]

        keys = JSONPath.get_target_keys(root, current_key)
        for key in keys:
            if len(prop_keys) == 1:
                result.append(Node(root, key))
            else:
                JSONPath._search_iterator(root[key], prop_keys[1:], result)

    @staticmethod
    def get_target_keys(root, key):
        # EXAMPLE
        # matched [0], [10], [*]
        is_list = re.match('\[(\d+|\*)\]', key)

        if not is_list:
            if isinstance(root, dict) and root.get(key):
                return (key,)
            else:
                return ()

        index = key.strip('[').strip(']')
        if not isinstance(root, list):
            return ()
        if index == '*':
            return range(0, len(root))
        if index.isdigit() and int(index) < len(root) - 1:
            return (int(index),)

        return ()


class Node:
    def __init__(self, parent, key):
        self.parent = parent
        self.node = parent[key]
        self.key = key

jsonpath = JSONPath()



from flask import redirect
from ....version import VERSION
from lyrebird import application
from flask_restful import Resource, request
from lyrebird.mock import context


class SanpshotImport(Resource):
    def get(self, url):
        application.snapshot_import_uri = url
        application.active_menu = {
            'name': 'datamanager',
            'title': 'DataManager',
            'type': 'router',
            'path': '/datamanager'
        }
        return redirect(f"/ui/?v={VERSION}#/datamanager/import")

    def post(self):
        parent_node = request.json.get("parentNode")
        snapshot_name = request.json.get('snapshotName', '')
        if "id" not in parent_node:
            return application.make_fail_response(msg="object has no attribute : parentNode.id")
        context.application.data_manager.import_snapshot(parent_node["id"], snapshot_name)
        return application.make_ok_response()


class SnapShotImportDetail(Resource):
    def get(self):
        snapshot_detail = context.application.data_manager.decompress_snapshot()['snapshot_detail']
        snapshot_detail.pop('children')
        return application.make_ok_response(data=snapshot_detail)
